<?php

/**
 * @category   Inchoo
 * @package    Inchoo_Cdyne
 * @author     Mladen Lotar - mladen.lotar@surgeworks.com
 */

class Inchoo_Cdyne_Model_Cdyne
{
	const MODULE_NAME = 'Inchoo_Cdyne';
	const CDYNE_API_KEY = 'inchoo/cdyne/inchoo_cdyne_api_key';

	static $_apiUri = 'http://pav3.cdyne.com/PavService.svc?wsdl';
	static $_SoapClient;

	static $errorMessage = "We cannot deliver to the address you provided. Please check if you've typed it incorrectly.";

	//Set to true for debugging
	static $testing = false;

	/*
	static call to SOAP API
	*/
	private static function _getSoapClient()
	{
		if(!is_null(self::$_SoapClient)){
			return self::$_SoapClient;
		}
		self::$_SoapClient = new SoapClient(self::$_apiUri);;
		return self::$_SoapClient;
	}

	/*
	Inject message into OneStepCheckout extension
	*/
	public function setCdyneMessage($event)
	{
		if (!$event) {
			return;
		}

		if('onestepcheckout.checkout' == $event->getEvent()->getBlock()->getNameInLayout()) {
			if (!Mage::getStoreConfig('advanced/modules_disable_output/'.self::MODULE_NAME)) {
				$errorValue = Mage::getSingleton('core/session')->getCdyneErrorMessage();
				if (!empty($errorValue)) {
					$event->getEvent()->getBlock()->formErrors['unknown_source_error'] = $errorValue;
					Mage::getSingleton('core/session')->setCdyneErrorMessage(null);
				}
			}
		}
	}

	/*
	Prepare data for API call
	*/
	private static function _preparePostDataAsParameters($postAddress = null)
	{
		$apiKey = Mage::getStoreConfig(self::CDYNE_API_KEY);
		
		$params = array(
			'FirmOrRecipient' => $postAddress['firstname'] . ' ' .$postAddress['lastname'],
			'PrimaryAddressLine' => $postAddress['street'][0],
			'SecondaryAddressLine' => $postAddress['street'][1],
			'Urbanization' => '',
			'CityName' => $postAddress['city'],
			'State' => $postAddress['region'],
			'ZipCode' => $postAddress['postcode'],
			'LicenseKey' => $apiKey
		);

		return $params;
	}

	/*
	Get response from API
	*/
	private static function _checkDataWithApi($postAddress)
	{
		$client = self::_getSoapClient();
		$params = self::_preparePostDataAsParameters($postAddress);
		$result = $client->VerifyAddress($params);
		$responseCode = $result->VerifyAddressResult->ReturnCode;

		return $responseCode;
	}

	/*
	Response code list:
	VerifyAddress Output
	1-Invalid input
	2- Invalid License Key
	10 – Input Address is not found
	100 – Input Address is DPV confirmed for all components
	101 – Input Address is found, but not DPV confirmed
	102 –Input Address Primary number is DPV confirmed. Secondary number is present but not DPV confirmed
	103 – Input address Primary number is DPV confirmed, Secondary number is missing.
	200 – Canadian address on input. Verified on City level only.
	*/

	public function checkAddress($event = null)
	{
		if (!$event) {
			return;
		}
		$request = Mage::app()->getRequest();

		//Allowed response codes
		$allowedResponseCodes = array(
			100 => 'Input Address is DPV confirmed for all components',
			101 => 'Input Address is found, but not DPV confirmed',
			102 => 'Input Address Primary number is DPV confirmed. Secondary number is present but not DPV confirmed',
			103 => 'Input address Primary number is DPV confirmed, Secondary number is missing.',
			200 => 'Canadian address on input. Verified on City level only.'
		);

		if ($request->isPost()) {

			$postAddress = $request->getPost();
			$postAddress = $postAddress['shipping'];

			if (is_array($postAddress)) {

				//Retrieve data from API
				//$responseCode = 2;
				$responseCode = self::_checkDataWithApi($postAddress);

				//Check if response is in allowed codes array
				if (in_array($responseCode, array_keys($allowedResponseCodes))) {
					//Check details and log accordingly
					if ($responseCode > 100) {
						//Log if there's something to warn about
						Mage::log(
								date("Y-m-d h:i:s") .
									' - ' .
									$allowedResponseCodes[$responseCode],
								Zend_Log::WARN,
								'Cdyne_Extension_Warnings.txt',
								true
						);
					}
				} else {
					Mage::log(
							date("Y-m-d h:i:s") .
								' - Invalid License Key',
							Zend_Log::CRIT,
							'Cdyne_Extension_Errors.txt',
							true
					);
				}

				//Check if response reports invalid address
				if ($responseCode == 10) {
					$errorValue = Mage::getSingleton('core/session')->getCdyneErrorMessage(self::$errorMessage);
					if (empty($errorValue)) {
						Mage::getSingleton('core/session')->setCdyneErrorMessage(self::$errorMessage);
					}

					header('Location: ' .Mage::getUrl('onestepcheckout/'));
					exit;
				} else {
					Mage::register('cdyneErrorMessage','',true);
				}

				if (self::$testing) {
					Zend_Debug::dump('Response code: ' . $responseCode);
					exit;
				}
			} else {
				$postVars = var_export($request->getPost(), true);
				Mage::log(
						'Problem with POST values parsing.
						POST variables:
						' . $postVars,
						Zend_Log::WARN,
						'Cdyne_Extension' . date("Y-m-d-h-i-s") . '.txt',
						true
				);
			}
		}
	}
}